package org.syncloud.webui;

import org.mortbay.jetty.Handler;
import org.mortbay.jetty.Server;
import org.mortbay.jetty.handler.HandlerList;
import org.mortbay.jetty.handler.ResourceHandler;
import org.mortbay.jetty.servlet.Context;
import org.mortbay.jetty.servlet.ServletHolder;
import org.syncloud.facade.ConfigurationManager;
import org.syncloud.facade.CoreFactory;
import org.syncloud.facade.Service;
import org.syncloud.webui.actions.*;
import org.syncloud.webui.json.StorageMetadataJsonWriter;

import javax.servlet.Servlet;
import java.util.HashMap;
import java.util.Map;

public class WebUiFactory {

    private static WebUiFactory instance;

    private Map<String, Servlet> servlets;
    private ResourceHandler resourceHandler;
    private Handler servletHandler;
    private Server server;
    private Service service;
    private final CoreFactory coreFactory;

    public WebUiFactory(CoreFactory factory) {
        coreFactory = factory;
    }

    public void init() {
        servlets = createServlets();
        servletHandler = createServletHandler(servlets);
        resourceHandler = createResourceHandler("root", "syncs.html");
        server = configureJettyServer();
        service = new WebServer(server, getPortProvider(coreFactory.getConfiguration()));
    }

    protected WebPortProvider getPortProvider(final ConfigurationManager configuration) {
        return new WebPortProvider() {
            @Override
            public int getPort() {
                return configuration.getConfig().getWebServerPort();
            }
        };
    }

    public Service getService() {
        return service;
    }

    public ResourceHandler getResourceHandler() {
        return resourceHandler;
    }

    public Handler getServletHandler() {
        return servletHandler;
    }

    private HashMap<String, Servlet> createServlets() {
        HashMap<String, Servlet> servlets = new HashMap<String, Servlet>();
        servlets.put("/syncs", new SyncListServlet(coreFactory.getSyncDao()));
        servlets.put("/syncs.save", new SyncSaveServlet(coreFactory.getSyncDao()));
        servlets.put("/storage.info", createStorageTypeServlet());
        servlets.put("/accounts", new AccountsServlet(coreFactory.getAccountsDao()));
        servlets.put("/account.add", new AccountAddServlet(coreFactory.getAuthenticationManager()));
        servlets.put("/account.delete", new AccountDeleteServlet(coreFactory.getAccountsDao()));
        servlets.put("/folders", new FolderListJsonServlet(coreFactory.getFoldersProxy(), coreFactory.getAuthenticationManager()));

        return servlets;
    }

    protected StorageTypeServlet createStorageTypeServlet() {
        return new StorageTypeServlet(new StorageMetadataJsonWriter(), coreFactory.getStorageManager().getMetadata());
    }

    private ResourceHandler createResourceHandler(String webappRoot, String indexPage) {
        ResourceHandler resourceHandler = new ResourceHandler();
        setResourceBase(resourceHandler, webappRoot);
        resourceHandler.setWelcomeFiles(new String[]{indexPage});
        return resourceHandler;
    }

    protected void setResourceBase(ResourceHandler resourceHandler, String webappRoot) {
        resourceHandler.setResourceBase(getClass().getClassLoader().getResource(webappRoot).toExternalForm());
    }

    private Handler createServletHandler(Map<String, Servlet> servlets) {
        Context context = new Context();
        for (Map.Entry<String, Servlet> entry : servlets.entrySet()) {
            context.addServlet(new ServletHolder(entry.getValue()), entry.getKey());
        }
        return context;
    }

    private Server configureJettyServer() {
        Server server = new Server();
        HandlerList handlerList = new HandlerList();

        handlerList.addHandler(getResourceHandler());
        handlerList.addHandler(getServletHandler());
        server.setHandler(handlerList);
        return server;
    }

}
